@@ -327,6 +327,8 @@ func (txn *Txn) modify(e *Entry) error {
 		return exceedsSize("Key", maxKeySize, e.Key)
 	case int64(len(e.Value)) > txn.db.opt.ValueLogFileSize:
 		return exceedsSize("Value", txn.db.opt.ValueLogFileSize, e.Value)
+	case txn.db.opt.InMemory && len(e.Value) > txn.db.opt.ValueThreshold:
+		return exceedsSize("Value", int64(txn.db.opt.ValueThreshold), e.Value)
 	}
 
 	if err := txn.checkSize(e); err != nil {
