@@ -412,6 +412,7 @@ func TestValueGC4(t *testing.T) {
 	err = kv.vlog.Close()
 	require.NoError(t, err)
 
+	kv.vlog.init(kv)
 	err = kv.vlog.open(kv, valuePointer{Fid: 2}, kv.replayFunction())
 	require.NoError(t, err)
 
@@ -642,8 +643,11 @@ func TestPartialAppendToValueLog(t *testing.T) {
 	checkKeys(t, kv, [][]byte{k3})
 	// Replay value log from beginning, badger head is past k2.
 	require.NoError(t, kv.vlog.Close())
-	require.NoError(t,
-		kv.vlog.open(kv, valuePointer{Fid: 0}, kv.replayFunction()))
+
+	kv.vlog.init(kv)
+	require.NoError(
+		t, kv.vlog.open(kv, valuePointer{Fid: 0}, kv.replayFunction()),
+	)
 	require.NoError(t, kv.Close())
 }
 
@@ -780,6 +784,9 @@ func TestPenultimateLogCorruption(t *testing.T) {
 	if db0.valueDirGuard != nil {
 		require.NoError(t, db0.valueDirGuard.release())
 	}
+	require.NoError(t, db0.vlog.Close())
+	require.NoError(t, db0.manifest.close())
+	require.NoError(t, db0.registry.Close())
 
 	opt.Truncate = true
 	db1, err := Open(opt)
@@ -799,7 +806,9 @@ func TestPenultimateLogCorruption(t *testing.T) {
 func checkKeys(t *testing.T, kv *DB, keys [][]byte) {
 	i := 0
 	txn := kv.NewTransaction(false)
+	defer txn.Discard()
 	iter := txn.NewIterator(IteratorOptions{})
+	defer iter.Close()
 	for iter.Seek(keys[0]); iter.Valid(); iter.Next() {
 		require.Equal(t, iter.Item().Key(), keys[i])
 		i++
